import { SynthesizedStack } from "./synth-stack";
import { CdktfStack, StackUpdate } from "./cdktf-stack";
import { TerraformPlan } from "./models/terraform";
declare type MultiStackApprovalUpdate = {
    type: "waiting for approval";
    stackName: string;
    plan: TerraformPlan;
    approve: () => void;
    dismiss: () => void;
    stop: () => void;
};
export declare type ProjectUpdate = {
    type: "synthesizing";
} | {
    type: "synthesized";
    stacks: SynthesizedStack[];
    errorMessage?: string;
} | StackUpdate | MultiStackApprovalUpdate;
export declare type SingleStackOptions = {
    stackName?: string;
};
export declare type MultipleStackOptions = {
    stackNames?: string[];
};
export declare type ExecutionOptions = MultipleStackOptions & {
    autoApprove?: boolean;
    ignoreMissingStackDependencies?: boolean;
};
export declare class CdktfProject {
    stacks?: SynthesizedStack[];
    hardAbort: () => void;
    private synthCommand;
    private outDir;
    private workingDirectory;
    private onUpdate;
    private onLog?;
    private abortSignal;
    private stacksToRun;
    private stopAllStacksThatCanNotRunWithout;
    constructor({ synthCommand, outDir, onUpdate, onLog, workingDirectory, }: {
        synthCommand: string;
        outDir: string;
        onUpdate: (update: ProjectUpdate) => void;
        onLog?: (log: {
            stackName: string;
            message: string;
        }) => void;
        workingDirectory?: string;
    });
    private stopAllStacks;
    private handleApprovalProcess;
    getStackExecutor(stack: SynthesizedStack, opts?: ExecutionOptions): CdktfStack;
    synth(): Promise<SynthesizedStack[]>;
    diff(opts?: SingleStackOptions): Promise<TerraformPlan | undefined>;
    deploy(opts?: ExecutionOptions): Promise<void>;
    destroy(opts?: ExecutionOptions): Promise<void>;
    fetchOutputs(opts?: SingleStackOptions): Promise<Record<string, any> | undefined>;
}
export {};
